package org.btcollider.cnc;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.TimeUnit;
import org.btcollider.cnc.comm.CommException;
import org.btcollider.cnc.comm.CommServer;
import org.btcollider.cnc.keysrv.KSFactory;
import org.btcollider.cnc.keysrv.KeyServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Hello world!
 *
 */
public class CnC {
  private static final Logger log = LoggerFactory.getLogger(CnC.class);

  public static final int PORT = 26765;
  // Invariant: positive long (2^63-1) must be able to hold all bits handled here
  public static final int MAX_BITS = 62;

  public static void main(String[] args) {
    if (args.length < 2 || args.length > 3) {
      System.out
          .println("USAGE: java -jar cnc.jar <succFile Path> <dumpFile Path> [loadDump Path]");
      System.exit(-1);
    }

    String succFile = args[0];
    String dumpFile = args[1];

    KeyServer kts = KSFactory.build(54, 30, 90, TimeUnit.MINUTES, 0.65f);
    if (args.length == 3) {
      log.info("Undumping from {}", args[2]);
      try (InputStream is = new FileInputStream(args[2])) {
        kts.undump(is);
      } catch (FileNotFoundException e) {
        log.error("Dump file {} not found", args[2], e);
      } catch (IOException e) {
        log.error("Undumping from {} failed", args[2], e);
      }
    }

    CommServer server = new CommServer(PORT, kts, succFile, dumpFile);
    try {
      server.listen();
    } catch (CommException e) {
      log.error("CommServer couldn't be started", e);
    } finally {
      server.stop();
    }
  }
}
